/*
 * Access peripherals on the I2C bus.
 *
 * Copyright (C) 2018, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/cpm-jz4730.h>
#include <l4/devices/i2c-jz4730.h>
#include <l4/devices/memory.h>

#include <stdio.h>



/* Scan the I2C bus by performing speculative reads from each device address. */

static void i2c_scan(void *i2c_channel)
{
  uint8_t buf[1];
  unsigned int address;

  for (address = 0; address < 0x20; address++)
    printf("%02x ", address);
  printf("\n");

  for (address = 0; address < 0x20; address++)
    printf("-- ");

  for (address = 0; address < 0x80; address++)
  {
    if ((address % 32) == 0)
      printf("\n");

    if (jz4730_i2c_read(i2c_channel, address, buf, 1))
      printf("%02x ", address);
    else
      printf("?? ");
  }

  printf("\n");
  for (address = 0; address < 0x20; address++)
    printf("-- ");
  printf("\n\n");
}

/* Interpret RTC registers. */

static void rtc_datetime(uint8_t *rtcregs)
{
  const char *weekdays[] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};

  printf("%s %d%d-%d%d-%d%d %d%d:%d%d:%d%d\n",
    weekdays[rtcregs[0x06] & 0x07],
    (rtcregs[0x08] & 0xf0) >> 4,
    rtcregs[0x08] & 0x0f,
    (rtcregs[0x07] & 0x10) >> 4,
    rtcregs[0x07] & 0x0f,
    (rtcregs[0x05] & 0x30) >> 4,
    rtcregs[0x05] & 0x0f,
    (rtcregs[0x04] & 0x30) >> 4,
    rtcregs[0x04] & 0x0f,
    (rtcregs[0x03] & 0x70) >> 4,
    rtcregs[0x03] & 0x0f,
    (rtcregs[0x02] & 0x70) >> 4,
    rtcregs[0x02] & 0x0f);
}



int main(void)
{
  void *cpm;
  void *i2c, *i2c0;

  /* Peripheral memory. */

  l4_addr_t cpm_base = 0, cpm_base_end = 0;
  l4_addr_t i2c_base = 0, i2c_base_end = 0;

  /* Obtain resource details describing I/O memory. */

  printf("Access CPM...\n");

  if (get_memory("jz4730-cpm", &cpm_base, &cpm_base_end) < 0)
    return 1;

  printf("CPM at 0x%lx...0x%lx.\n", cpm_base, cpm_base_end);

  printf("Access I2C...\n");

  if (get_memory("jz4730-i2c", &i2c_base, &i2c_base_end) < 0)
    return 1;

  printf("I2C at 0x%lx...0x%lx.\n", i2c_base, i2c_base_end);

  /* Obtain CPM and I2C references. */

  cpm = jz4730_cpm_init(cpm_base);
  i2c = jz4730_i2c_init(i2c_base, i2c_base_end, cpm, 100000); /* 100 kHz */
  i2c0 = jz4730_i2c_get_channel(i2c, 0);

  /* Enable I2C. */

  jz4730_i2c_enable(i2c0);

  uint8_t buf[32];
  unsigned int nwritten;

  /* Set the clock: 12:34:56 on Saturday 2nd January 2021. */

  buf[0] = 2; buf[1] = 0x56; buf[2] = 0x34; buf[3] = 0x12; buf[4] = 0x02; buf[5] = 0x06; buf[6] = 0x01; buf[7] = 0x21;
  nwritten = jz4730_i2c_write(i2c0, 0x51, buf, 8);

  printf("Written: %d\n", nwritten);

  /* Issue selection of device register 0. */

  buf[0] = 0;
  nwritten = jz4730_i2c_write(i2c0, 0x51, buf, 1);

  printf("Written: %d\n", nwritten);

  /* Read the contents of 16 registers. */

  unsigned int nread = jz4730_i2c_read(i2c0, 0x51, buf, 16);

  printf("Read: %d\n", nread);
  for (unsigned int i = 0; i < nread; i++)
    printf("%02x ", buf[i]);
  printf("\n");

  /* Show the interpreted date and time. */

  rtc_datetime(buf);

  /* Scan the bus. */

  printf("Scan I2C0...\n");
  i2c_scan(i2c0);

  return 0;
}
